from textwrap import dedent
from unittest.mock import (
    call,
    mock_open,
    patch,
)

from testtools import TestCase

import hooks


class MockConfig(dict):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._changed = set()

    def _mark_changed(self, key):
        self._changed.add(key)

    def changed(self, key):
        return key in self._changed


class RsyncTest(TestCase):

    def add_patch(self, *args, **kwargs):
        p = patch(*args, **kwargs)
        self.addCleanup(p.stop)
        return p.start()

    def setUp(self):
        super(RsyncTest, self).setUp()

        self.config_get = self.add_patch("hooks.config_get")
        self.open = mock_open()
        self.add_patch("hooks.open", self.open, create=True)
        self.exists = self.add_patch("os.path.exists")
        self.unlink = self.add_patch("os.unlink")
        self.service_restart = self.add_patch("hooks.host.service_restart")

    def test_with_no_log_hosts_allow_and_no_existing_config(self):
        self.config_get.return_value = {"log_hosts_allow": ""}
        self.exists.return_value = False

        hooks.write_rsync_config()

        self.open.assert_not_called()
        self.unlink.assert_not_called()
        self.service_restart.assert_not_called()

    def test_with_no_log_hosts_allow_and_existing_config(self):
        self.config_get.return_value = {"log_hosts_allow": ""}
        self.exists.return_value = True

        hooks.write_rsync_config()

        self.open.assert_not_called()
        self.unlink.assert_called_once_with("/etc/rsync-juju.d/010-squid.conf")
        self.service_restart.assert_called_once_with("rsync")

    def test_with_log_hosts_allow(self):
        config_data = MockConfig(log_hosts_allow="bastion.example")
        config_data._mark_changed("log_hosts_allow")
        self.config_get.return_value = config_data

        hooks.write_rsync_config()

        rsync_config_contents = dedent("""
            [squid-logs]
              path = /var/log/squid
              comment = Squid Logs
              list = false
              read only = true
              hosts allow = bastion.example
            """)
        self.open.assert_has_calls([
            call("/etc/rsync-juju.d/010-squid.conf", "w"),
            call().__enter__(),
            call().write(rsync_config_contents),
            call().__exit__(None, None, None),
        ])
        self.unlink.assert_not_called()
        self.service_restart.assert_called_once_with("rsync")

    def test_with_unchanged_log_hosts_allow(self):
        config_data = MockConfig(log_hosts_allow="bastion.example")
        self.config_get.return_value = config_data

        hooks.write_rsync_config()

        self.open.assert_not_called()
        self.unlink.assert_not_called()
        self.service_restart.assert_not_called()
