#! /usr/bin/python3

# Copyright 2020-2021 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

"""Link system-installed Python modules into a virtualenv."""

from __future__ import absolute_import, print_function, unicode_literals

from argparse import ArgumentParser
from distutils.sysconfig import get_python_lib
import importlib
import os.path
import subprocess


def link_module(name, virtualenv_libdir):
    module = importlib.import_module(name)
    path = module.__file__
    if os.path.basename(path).startswith("__init__."):
        path = os.path.dirname(path)
    system_libdir = get_python_lib(plat_specific=path.endswith(".so"))
    if os.path.commonprefix([path, system_libdir]) != system_libdir:
        raise RuntimeError(
            "%s imported from outside %s (%s)" % (name, system_libdir, path))
    target_path = os.path.join(
        virtualenv_libdir, os.path.relpath(path, system_libdir))
    if os.path.lexists(target_path) and os.path.islink(target_path):
        os.unlink(target_path)
    os.symlink(path, target_path)


def main():
    parser = ArgumentParser()
    parser.add_argument("virtualenv")
    parser.add_argument("modules", metavar="module", nargs="+")
    args = parser.parse_args()

    virtualenv_libdir = subprocess.check_output(
        [os.path.join(args.virtualenv, "bin", "python"), "-c",
         "from distutils.sysconfig import get_python_lib; "
         "print(get_python_lib())"],
        universal_newlines=True).rstrip("\n")

    for name in args.modules:
        link_module(name, virtualenv_libdir)


if __name__ == "__main__":
    main()
